#--------------------------------------------------------------------------------------------------------
findRankK:  
	#   $a0 holds k,  the index of the desired element
	#   $a1 is the base address of list  (list is size L words, buffer of size L words 
	#    - so array is of size 2L words)
	#   $a2 holds the length L of the list
	#
	#   $s0 and $s1 are low and high indices in the partitionedList.  
	#   They hold next available slot of L1 and L3, respectively.
	#   $s2 is the base address of the  partitioned list [L1,L2,L3]
	#   $s3 will be an index i into the list;  initialize it to 1
	#   $s4 will hold the pivot  

	addi	$sp, $sp, -4			#  remember return address
	sw	$ra, 0($sp)			#  (need to return to caller -- either recursive, or level 0 parent)

	addi	$sp, $sp, -20			#  remember values of the save registers we will use
	sw	$s0, 0($sp)		
	sw	$s1, 4($sp)		
	sw	$s2, 8($sp)		
	sw	$s3, 12($sp)		
	sw	$s4, 16($sp)				

	#  compute base address of partitioned list [L1,L2,L3] 
	
	move	$t0,  $a2                       #  $a2 is L  (length of the list)
	sll	$t0,  $t0, 2
	add	$s2, $a1, $t0 	                #  $a1 is base address of list
	
# ------------- partition into L1, L2, L3 -------------------------- 
	
	addi 	$s0, $0, 0         		#  low = 0
	lw	$s4, 0($a1)			#  pivot = list[0]
	addi 	$s3, $0, 1			#  i = 1
	addi	$s1, $a2, -1			#  high = L -1	
	
#     take one pass through the list and copy elements into the buffer 
	
whilei:	
	bge	$s3, $a2, endPartitioning		#  i >= L  then partitioning into L1, L2, L3 finished
						#  else....
						
						#  $t0  is  a byte offset used for indexing the list or partitioned list
						#  $t1  is list[i]
	sll	$t0, $s3, 2			   
	add	$t0, $a1, $t0 	 		#  compute address of list[i]
	lw	$t1, 0($t0)			#  load list[i]
	bge	$t1, $s4, notinL1           	#  branch if list[i] >= pivot
	sll	$t0, $s0, 2			#  else
	add	$t0, $s2, $t0 			#  compute address of partitionedList[low]						
	sw	$t1, 0($t0)			#  partitionedList[low] = list[i]
	addi	$s0, $s0, 1			#  low++
	j	endWhilei	
	
	# reach here if the current element is not less than the pivot  
	# i.e. current element is greater than or equal to the pivot
	
notinL1:		
	#  if list[i] == pivot,  then branch
	#  else list[i] > pivot and so continue
	
	beq	$t1, $s4, endWhilei           	#  branch if list[i] == pivot
	sll	$t0, $s1, 2			#  else
	add	$t0, $s2, $t0 			#  compute address of partitionedList[high]						
	sw	$t1, 0($t0)   			#  partitionedList[high] = list[i]
	addi 	$s1, $s1, -1			#  high--

	#  increment i and go back to start of loop
endWhilei:	
	addi	$s3, $s3, 1			#  i++
	j	whilei
	
endPartitioning:

#  Don't bother copying the pivot elements into L2.  This is unnecessary as we'll see below.

# --------- done partitioning L into L1, L2, L3 -------------------------

#  Copy either L1 or L3 back to list part of buffer (unless pivot is in L2, in which case we are done)
#  At this point,  low ($s0) is the number of elements in L1.

	bge	$a0, $s0, kgeLow		#  if k >= low, then pivot in L2 or L3, so branch,  
						#  else pivot in L1
# case pivot is in L1

	addi	$s3, $0, 0			#  i = 0
whileL1copylist:
	bge	$s3, $s0, endwhileL1copylist	#  branch if i == low (use >= condition for debugging)
	sll	$t0, $s3, 2
	add	$t1, $a1, $t0			#  compute address of list[i]
	add	$t2, $s2, $t0			#  compute address of partitionedList[i]
	lw	$t3, 0($t2)
	sw	$t3, 0($t1)			#  list[i] = partitionedList[i]
	addi	$s3, $s3, 1			#  i++
	j	whileL1copylist
endwhileL1copylist:

	#  Prepare for a recursive call.  
	#  We don't need to save the $a2 register (length of current light)

	addi	$t1, $s0, 0 			#  $t1 initialized to low 
	move 	$a2, $t1			#  lengthlist = low
	 
jal1:						#  tail recursion
	jal	findRankK			#  a0 hasn't changed.  we still want item k.
	j	return_findRankK                

#  We reach here if the pivot is not in L1.

kgeLow:
	bgt	$a0, $s1, kgtHigh		#  if (k > high), pivot is in L3, so branch 

#  Otherwise, the pivot is in L2 so we can return it.

	move	$v0, $s4		
	j	return_findRankK

#  Deal with case that pivot is in L3
#  Copy L3 back to the base.

kgtHigh:
	#  first figure out the rank that will be used in the recursive call
	
	addi	$s3, $0, 0			#  i = 0 
	addi	$t4, $s1, 1			#  j = high+1   (first element in L3)
						#  We will want item of rank k - (high+1) from L3
	sub	$a0, $a0, $t4			#  So, a0 needs to change.   

whileL3copylist:
	beq	$t4, $a2, endwhileL3copylist	#  if (j == L) then done copying list
	sll	$t0, $s3, 2                     #  else
	add	$t1, $a1, $t0			#  compute address of list[i]
	sll	$t5, $t4, 2
	add	$t2, $s2, $t5			#  compute address of partitionedList[j]
	lw	$t3, 0($t2)
	sw	$t3, 0($t1)			#  list[i] = partitionedList[j]
	addi	$s3, $s3, 1			#  i++
	addi	$t4, $t4, 1			#  j++
	j	whileL3copylist
endwhileL3copylist:
			
	move 	$a2, $s3			#  the length of the L3 (L = i)
jal2:				
	jal	findRankK			#  the new rank argument ($a0) was found above 
	
return_findRankK:
	
	#  restore values of save registers we used
	
	lw	$s0, 0($sp)		
	lw	$s1, 4($sp)		
	lw	$s2, 8($sp)		
	lw	$s3, 12($sp)		
	lw	$s4, 16($sp)				
	addi	$sp, $sp, 20 			

	lw	$ra, 0($sp)			#  restore return address
	addi	$sp, $sp, 4			#  restore stack pointer

	jr	$ra
	 
end_findRank:
